/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#ifdef __linux__
#include <linux/vt.h>
#endif
#include <string.h>
#include <ggi/libggi.h>
#include "ggi_screensaver.h"

#include <structs.h>

/* register screensavers */
void reg_savers();
void done_savers();
void ge_stars(ggi_visual_t visual);

/*
 * This structure contains all screensavers in this module:
 */
ggi_saver_list saver_list[] = {
	{ ge_stars, "Starfield"},
};

/*
 * register all screen-savers in the main module
 */
void reg_savers()
{
	int i;
	for (i=0; i<(sizeof(saver_list)/sizeof(ggi_saver_list)); i++) {
		install_saver(saver_list[i].func, saver_list[i].name);
	}
}

/*
 * free resources allocated by the screen-savers
 */
void done_savers()
{
}  

#define STARANZ 100

double rand01(void)
{ return (rand()&0xffff)/(double)0x10000; }

void ge_stars(ggi_visual_t visual)
{ 
  int x1,y,z;

  typedef struct { double x,y,z; } pix3d;
  typedef struct { int x1,y; } pix2d;
  pix3d pix[STARANZ],*pp;
  pix2d oldpos[STARANZ];
  double da,nda,dz,ndz,cx,sx,cz,sz,z0,hlp;

	ggi_mode mode;
	int xsize, ysize;
	ggiGetMode(visual,&mode);
	xsize=mode.visible.x;
	ysize=mode.visible.y;

  z0=2.0*ysize;
  memset(pix,0,sizeof(pix));
  for(z=0;z<STARANZ;z++) pix[z].z=-z0+50;
  memset(oldpos,0,sizeof(oldpos));

  y=0;da=nda=0.0;dz=ndz=0.0;
  while(!switchaway&&!ggiKbhit(visual))
  { if (rand01()<0.001) nda=(rand01()-0.5)/200.0;
    da=(nda+999.0*da)/1000.0;
    cx=cos(da);sx=sin(da);
    if (rand01()<0.001) ndz=(rand01()-0.5)/200.0;
    dz=(ndz-10*nda+989.0*dz)/1000.0;
    cz=cos(dz/10);sz=sin(dz/10);
    for(z=0;z<STARANZ;z++)
    { 
      ggiSetGCForeground(visual,0);
      ggiDrawPixel(visual,oldpos[z].x1,oldpos[z].y);

      pp=&pix[z];
      retry:
      pp->z-=5.0;
      hlp  =pp->x* cx+pp->y*sx;
      pp->y=pp->x*-sx+pp->y*cx;
      pp->x=hlp;
      hlp  =pp->x* cz+pp->z*sz;
      pp->z=pp->x*-sz+pp->z*cz;
      pp->x=hlp;
      hlp=z0/(pp->z+z0);
      x1=xsize/2+pp->x*hlp;
      y=ysize/2+pp->y*hlp;
      
      if (pp->z<-z0+100.0 || x1<10 || x1>=xsize-10 
                          || y <10 || y >=ysize-10 ) 
      { pp->x=rand01()*z0*2-1*z0;pp->y=rand01()*2*z0-1*z0;
        pp->z=rand01()*z0*10+z0;goto retry;
      }
      ggiSetGCForeground(visual,128);
      ggiDrawPixel(visual,oldpos[z].x1=x1,oldpos[z].y=y);
    }
/*   usleep(1);*/
  }
}
